package de.meinefirma.meinprojekt.ftp;

import java.util.Properties;
import org.junit.Assert;
import org.junit.Test;

public class ExtractFtpParmsFromUrlTest
{
   @Test
   public void testExtractFtpParmsFromUrl()
   {
      Assert.assertNull( FtpFileWrapper.extractFtpParmsFromUri( null ) );
      Assert.assertNull( FtpFileWrapper.extractFtpParmsFromUri( "" ) );
      Assert.assertNull( FtpFileWrapper.extractFtpParmsFromUri( " " ) );
      Assert.assertNull( FtpFileWrapper.extractFtpParmsFromUri( "ftp://" ) );
      Assert.assertNull( FtpFileWrapper.extractFtpParmsFromUri( "ftp:// " ) );
      Assert.assertNull( FtpFileWrapper.extractFtpParmsFromUri( "ftp:///r" ) );
      Assert.assertNull( FtpFileWrapper.extractFtpParmsFromUri( "ftp://:21/r" ) );
      Assert.assertNull( FtpFileWrapper.extractFtpParmsFromUri( "ftp://u:p@/r" ) );
      Assert.assertNull( FtpFileWrapper.extractFtpParmsFromUri( "ftp://u:p@:21/r" ) );
      Properties p;
      p = FtpFileWrapper.extractFtpParmsFromUri( "ftp://u:p@h:2121/r" );
      Assert.assertEquals( "usr=u, pwd=p, host=h, port=2121, root=/r", getFtpPropsAsString( p ) );
      p = FtpFileWrapper.extractFtpParmsFromUri( " ftp://u:p@h:2121/r " );
      Assert.assertEquals( "usr=u, pwd=p, host=h, port=2121, root=/r", getFtpPropsAsString( p ) );
      p = FtpFileWrapper.extractFtpParmsFromUri( "ftp://u:p@h/r:2121" );
      Assert.assertEquals( "usr=u, pwd=p, host=h, port=21, root=/r:2121", getFtpPropsAsString( p ) );
      p = FtpFileWrapper.extractFtpParmsFromUri( "ftp://h" );
      Assert.assertEquals( "usr=null, pwd=null, host=h, port=21, root=null", getFtpPropsAsString( p ) );
      p = FtpFileWrapper.extractFtpParmsFromUri( "ftp://h/" );
      Assert.assertEquals( "usr=null, pwd=null, host=h, port=21, root=/", getFtpPropsAsString( p ) );
      p = FtpFileWrapper.extractFtpParmsFromUri( "ftp://h:2121" );
      Assert.assertEquals( "usr=null, pwd=null, host=h, port=2121, root=null", getFtpPropsAsString( p ) );
      p = FtpFileWrapper.extractFtpParmsFromUri( "ftp://h:2121/" );
      Assert.assertEquals( "usr=null, pwd=null, host=h, port=2121, root=/", getFtpPropsAsString( p ) );
      p = FtpFileWrapper.extractFtpParmsFromUri( "ftp://h/r" );
      Assert.assertEquals( "usr=null, pwd=null, host=h, port=21, root=/r", getFtpPropsAsString( p ) );
      p = FtpFileWrapper.extractFtpParmsFromUri( "ftp://h:2121/r" );
      Assert.assertEquals( "usr=null, pwd=null, host=h, port=2121, root=/r", getFtpPropsAsString( p ) );
      p = FtpFileWrapper.extractFtpParmsFromUri( "ftp://@h:2121/r" );
      Assert.assertEquals( "usr=null, pwd=null, host=h, port=2121, root=/r", getFtpPropsAsString( p ) );
      p = FtpFileWrapper.extractFtpParmsFromUri( "ftp://u@h:2121/r" );
      Assert.assertEquals( "usr=u, pwd=null, host=h, port=2121, root=/r", getFtpPropsAsString( p ) );
      p = FtpFileWrapper.extractFtpParmsFromUri( "ftp://u:@h:2121/r" );
      Assert.assertEquals( "usr=u, pwd=null, host=h, port=2121, root=/r", getFtpPropsAsString( p ) );
      p = FtpFileWrapper.extractFtpParmsFromUri( "ftp://:p@h:2121/r" );
      Assert.assertEquals( "usr=null, pwd=p, host=h, port=2121, root=/r", getFtpPropsAsString( p ) );
   }

   private static String getFtpPropsAsString( Properties p )
   {
      if( p == null ) { return null; }
      return "usr="  + p.getProperty( FtpFileWrapper.FTP_USR_KEY  ) +
           ", pwd="  + p.getProperty( FtpFileWrapper.FTP_PWD_KEY  ) +
           ", host=" + p.getProperty( FtpFileWrapper.FTP_HOST_KEY ) +
           ", port=" + p.getProperty( FtpFileWrapper.FTP_PORT_KEY ) +
           ", root=" + p.getProperty( FtpFileWrapper.FTP_PATH_KEY );
   }
}
