package de.meinefirma.meinprojekt.ftp;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.List;

import org.apache.ftpserver.FtpServer;
import org.apache.ftpserver.ftplet.FtpException;
import org.junit.AfterClass;
import org.junit.Assert;
import org.junit.BeforeClass;
import org.junit.Test;

import de.meinefirma.meinprojekt.ftp.FtpFileWrapper;

/**
 * Siehe unbedingt die Erlaeuterungen zu:
 * {@link FtpFileWrapperVielParallelTest}
 */
public class FtpFileWrapperVielKopierenTest
{
   private static final int    FTP_PORT        = 2121;
   private static final String FTP_HOST        = "localhost";
   private static final String FTP_HOME_DIR    = "target/FtpHome";
   private static final String READ_USER_NAME  = "ReadUserName";
   private static final String READ_USER_PWD   = "ReadUserPwd";
   private static final String WRITE_USER_NAME = "WriteUserName";
   private static final String WRITE_USER_PWD  = "WriteUserPwd";
   private static final String FS_HOME_DIR     = "target/FileserverHome";
   private static final int    MAX_LOGINS      = 100;
   private static final int    MAX_TASKS_JUNIT = 10;
   private static final int    MAX_TASKS_MAIN  = 1000;
   private static final int    MAX_WARTENDPORT = 1900;
   private static FtpServer    ftpServer;

   /** Manueller Test mit externem FTP-Server */
   public static void main( String[] args )
   {
      // Maximal vollstaendige Kommandozeile:
      // ftp://WriteUserName:WriteUserPwd@localhost:4221 dir1 dir2 1000 1900 false
      String uri                  = ( args.length > 0 ) ? args[0] : "ftp://WriteUserName:WriteUserPwd@localhost:4221";
      String dstDir1              = ( args.length > 1 ) ? args[1] : "dir1";
      String dstDir2              = ( args.length > 2 ) ? args[2] : "dir2";
      String maxTasks             = ( args.length > 3 ) ? args[3] : "" + MAX_TASKS_MAIN;
      String maxWartendPort       = ( args.length > 4 ) ? args[4] : "" + MAX_WARTENDPORT;
      String useOnlyExistingFiles = ( args.length > 5 ) ? args[5] : "false";
      testVielKopieren( uri, true, dstDir1, dstDir2,
            parseInt( maxTasks, -1 ), parseInt( maxWartendPort, -1 ), parseBoolean( useOnlyExistingFiles ), true );
   }

   @BeforeClass
   public static void startFtpServer() throws FtpException, IOException
   {
      ftpServer = FtpTestUtil.createFtpServer( FTP_PORT, FTP_HOME_DIR,
            READ_USER_NAME, READ_USER_PWD, WRITE_USER_NAME, WRITE_USER_PWD, null, MAX_LOGINS );
      ftpServer.start();
   }

   @AfterClass
   public static void stoppFtpServer()
   {
      ftpServer.stop();
      ftpServer = null;
   }

   /** Teste Fileserver-Variante */
   @Test
   public void testFilesystem()
   {
      testVielKopierenJUnit( FS_HOME_DIR, false, MAX_TASKS_JUNIT );
   }

   /** Teste FTP-Variante */
   @Test
   public void testFtp()
   {
      testVielKopierenJUnit( "ftp://" + WRITE_USER_NAME + ":" + WRITE_USER_PWD + "@" + FTP_HOST + ":" + FTP_PORT, true, MAX_TASKS_JUNIT );
   }

   /** Interne gemeinsame FtpFileWrapper-Testmethode sowohl fuer FTP- als auch FS-Betrieb */
   private static void testVielKopierenJUnit( String uri, boolean isFtp, int maxTasks )
   {
      testVielKopieren( uri, isFtp, "target/dst6", "target/dst7", maxTasks, -1, false, false );
   }

   /** Interne gemeinsame VielKopieren-Testmethode sowohl fuer FTP- als auch FS-Betrieb */
   protected static void testVielKopieren( String uri, boolean isFtp, String dstDir1, String dstDir2,
         int maxTasks, int maxWartendPort, boolean useOnlyExistingFiles, boolean showInfos )
   {
      int i = Math.max( uri.indexOf( '/' ), uri.indexOf( '@' ) );
      String uriOhnePwd = ( isFtp && i > 0 ) ? uri.substring( ++i ) : uri;
      if( showInfos ) {
         String s = "url = " + uriOhnePwd + ", dir1 = " + dstDir1 + ", dir2 = " + dstDir2 + ", maxTasks = " + maxTasks;
         if( maxWartendPort >= 0 ) { s += ", maxWartendPort = " + maxWartendPort; }
         if( useOnlyExistingFiles ) { s += ", useOnlyExistingFiles"; }
         System.out.println( s );
         zeigeAnzahlNochWartenderPortsLokalUnterWindows();
      }
      String charEnc  = "UTF-8";
      String meinText = "BlaBlupp\r\n\u20AC"; // \u20AC = Euro-Zeichen
      String homeDir  = ( isFtp ) ? FTP_HOME_DIR : FS_HOME_DIR;
      (new File( homeDir + "/" + dstDir1 )).mkdirs();

      FtpFileWrapper ffwrapSource = FtpFileWrapper.createFtpFileWrapperFromUri( uri );
      FtpFileWrapper ffwrapResult = FtpFileWrapper.createFtpFileWrapperFromUri( uri );
      Assert.assertNotNull( "ffwrap null", ffwrapSource );
      Assert.assertNotNull( "ffwrap null", ffwrapResult );
      long[] countAndSizeSumVorher  = new long[0];
      long[] countAndSizeSumNachher = null;
      if( showInfos ) {
         countAndSizeSumVorher = ffwrapSource.getFileCountAndSizeSum( null, null );
         System.out.println( "Anzahl Dateien:              " + countAndSizeSumVorher[0] );
         System.out.println( "Summe Groesse:               " + countAndSizeSumVorher[1] );
      }
      List<String> files = null;
      if( useOnlyExistingFiles ) {
         try { files = ffwrapSource.listFileNames( dstDir1 ); } catch( IOException e ) {/*ok*/}
      }
      String err = null;
      int n = 0;
      long startZeit = System.nanoTime();
      try {
         while( n < maxTasks ) {
            if( maxWartendPort >= 0 ) {
               // Maximal 200 Sekunden warten:
               int wp, wn = 20;
               while( (wp = ermittleAnzahlNochWartenderPortsLokalUnterWindows()) > maxWartendPort && wn-- > 0 ) {
                  if( showInfos ) { System.out.println( "warten, weil Anzahl 'Wartender Ports' " + wp + " > " + maxWartendPort + " ..." ); }
                  Thread.sleep( 10000 );
               }
            }
            String srcDatei, dstDatei;
            if( useOnlyExistingFiles ) {
               // Verschiebe vorhandene Dateien:
               if( files == null || files.size() == 0 ) {
                  System.out.println( "Keine zu verschiebene Datei vorhanden in '" + dstDir1 + "'." );
                  break;
               }
               if( n >= files.size() ) {
                  break;
               }
               srcDatei = dstDir1 + "/" + files.get( n );
               dstDatei = dstDir2 + "/" + files.get( n++ );
            } else {
               // Verschiebe 1.txt, 2.txt, ... (entweder neu oder vorhanden)
               String datei = ++n + ".txt";
               srcDatei = dstDir1 + "/" + datei;
               dstDatei = dstDir2 + "/" + datei;
               // homeDir funktioniert nur im JUnit-Test und nicht mit main()-Methode:
               deleteFileIfExists( homeDir + "/" + dstDatei );
               // Quelldateien nur anlegen, falls sie noch nicht existieren,
               // dadurch koennen auch groessere Dateien manuell kopiert und verwendet werden:
               if( !ffwrapSource.existsFile( srcDatei ) ) {
                  ffwrapSource.storeFile( new ByteArrayInputStream( meinText.getBytes( charEnc ) ), srcDatei );
               }
            }
            FtpFileWrapper.moveRemoteFile( ffwrapSource, srcDatei, ffwrapResult, dstDatei );
            double dauer = ((System.nanoTime() - startZeit) / 10000000 / 100.);
            if( showInfos ) { System.out.println( n + " Dateien erstellt und verschoben in " + dauer + " s." ); }
         }
         if( showInfos ) {
            countAndSizeSumNachher = ffwrapSource.getFileCountAndSizeSum( null, null );
            zeigeAnzahlNochWartenderPortsLokalUnterWindows();
            System.out.println( "Anzahl Dateien:              " + countAndSizeSumNachher[0] );
            System.out.println( "Summe Groesse:               " + countAndSizeSumNachher[1] );
            if( useOnlyExistingFiles ) {
               Assert.assertEquals( "countAndSizeSum[0]", countAndSizeSumVorher[0], countAndSizeSumNachher[0] );
               Assert.assertEquals( "countAndSizeSum[0]", countAndSizeSumVorher[1], countAndSizeSumNachher[1] );
            }
         }
      } catch( Exception ex ) {
         err = ex.getMessage();
         err = ( err != null && err.trim().length() > 0 ) ? err : ex.toString();
         System.out.println( "Abbruch mit Fehlermeldung: " + err );
      } finally {
         ffwrapSource.logout();
         ffwrapResult.logout();
      }
   }

   static boolean deleteFileIfExists( String file )
   {
      File f = new File( file );
      return( f.exists() ) ? f.delete() : true;
   }

   static void zeigeAnzahlNochWartenderPortsLokalUnterWindows()
   {
      try {
         int i = ermittleAnzahlNochWartenderPortsLokalUnterWindows();
         if( i >= 0 ) {
            System.out.println( "Anzahl noch wartender Ports: " + i );
         }
      } catch( Exception e ) {
         throw new RuntimeException( e );
      }
   }

   static int ermittleAnzahlNochWartenderPortsLokalUnterWindows() throws IOException, InterruptedException
   {
      String osName = System.getProperty( "os.name" );
      if( osName != null && osName.toLowerCase().indexOf( "windows" ) >= 0 ) {
         Process proc = Runtime.getRuntime().exec( "cmd /c netstat -a | find /C \"WARTEND\"" );
         proc.waitFor();
         InputStream is = proc.getInputStream();
         BufferedReader in = new BufferedReader( new InputStreamReader( is, "UTF-8" ) );
         String s = in.readLine();
         return parseInt( s, -1 );
      }
      return -1;
   }

   static int parseInt( String s, int returnIfNotParsable )
   {
      try { return Integer.parseInt( s.trim() ); } catch( Exception e ) { return returnIfNotParsable; }
   }

   static boolean parseBoolean( String s )
   {
      return s != null && s.trim().equalsIgnoreCase( "true" );
   }
}
